/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com
 *
 * Derived from SWT addressBook example.

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

 *******************************************************************************/
package com.monblocnotes.review;

import java.sql.*;
//import java.util.ArrayList;
//import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.Random;
import java.util.ResourceBundle;

public class DataAccess {

	//private static ResourceBundle resReview = ResourceBundle.getBundle("review");

	static final int DA_SUCCESS = 0;
	static final int DA_LOADERR = -1;
	static final int DA_BOOTERR = -2;
	static final int DA_DOWNERR = -3;
	static final int DA_CONNERR = -4;
	static final int DA_LESSONSERR = -5;
	static final int DA_COMMITERR = -6;
	static final int DA_TRANSERR = -7;
	static final int DA_PAIRDELERR = -8;
	static final int DA_WORDTOOLONG = -9;
	static final int DA_PAIRINSERTERR = -10;
	static final int DA_GETRANDOMERR = -11;
	static final int DA_NOROWERR = -12;
	
	//private String driver = "org.apache.derby.jdbc.EmbeddedDriver";
	private String dbName="review";
	private String connectionURL = "jdbc:derby:" + dbName + ";create=true";
	private static Connection conn = null;
	private Statement s = null;
	private ResultSet resSet = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	
	// Object used to randomly select words pairs.
	private Random pairsRandom = new Random();
	
	static public String filterOutQuotes(String stringIn) {
		int l = stringIn.length();
		String stringOut = new String();
		for(int i = 0; i < l; i++) {
			if (stringIn.charAt(i) == '\'') {
				stringOut = stringOut + '\'';
			}
			stringOut = stringOut + stringIn.charAt(i);
		}
		return stringOut;
	}

	public int startDataAccess() {
		
		int retStatus = DA_SUCCESS;
		String tableName = null;
		boolean tablesExist = false;
		
		// Load driver. Not required on Java SE 6 or later.
		/*
		try {
		    Class.forName(driver).newInstance(); 
		} catch(java.lang.ClassNotFoundException e) {
		    System.out.println(e.getMessage());
		    retStatus = DA_LOADERR;
		} catch(java.lang.IllegalAccessException e) {
			System.out.println(e.getMessage());
		    retStatus = DA_LOADERR;
		} catch(java.lang.InstantiationException e) {
			System.out.println(e.getMessage());
		    retStatus = DA_LOADERR;
		}
		*/
		// Boot database.
		try {
		    conn = DriverManager.getConnection(connectionURL);
		}  catch (Throwable e)  {   
		   System.out.println(e.getMessage());
		   retStatus = DA_BOOTERR;
		}
		// Check whether we need to create database tables.
		try {
			s = conn.createStatement();
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			return DA_CONNERR;
		}

		// At this point, no connection error. No auto commit.
		try {
			conn.setAutoCommit(false);
		} catch (SQLException e1) {
			System.out.println(e1.getMessage());
			return DA_COMMITERR;
		}

		// Look for PAIRS table.
		try {
			resSet = s.executeQuery("SELECT * FROM SYS.SYSTABLES");
			while (resSet.next()) {
				tableName = resSet.getString(2);
				System.out.println("table name: " + tableName);
				if (tableName.contentEquals("PAIRS")) {
					tablesExist = true;
					System.out.println("table PAIRS already exists");
					break;
				}
			}
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}
		if (!tablesExist) {
			// Create tables.
			try {
				s.execute("CREATE TABLE PAIRS " +
						  "(WORDLANGUAGEA VARCHAR(128) NOT NULL, " +
						  " WORDLANGUAGEB VARCHAR(128) NOT NULL, " +
						  " PRIMARY KEY (WORDLANGUAGEA, WORDLANGUAGEB))");
				System.out.println("table PAIRS created");
				s.execute("CREATE INDEX WLAINDX ON PAIRS (WORDLANGUAGEA)");
				s.execute("CREATE INDEX WLBINDX ON PAIRS (WORDLANGUAGEB)");
				System.out.println("indices created on table PAIRS");
			} catch (SQLException e) {
				System.out.println(e.getMessage());				
			}
		}
		//
		return retStatus;
	}
	
	public int stopDataAccess() {
		int retStatus = DA_SUCCESS;
		boolean gotSQLExc = false;
		try {
			resSet.close();
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}
		try {
			s.close();
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}
		try {
			DriverManager.getConnection("jdbc:derby:;shutdown=true");
		} catch (SQLException se)  {
			if (se.getSQLState().equals("XJ015")) {
		         gotSQLExc = true;
			}
		}
		if (!gotSQLExc) {
			System.out.println("Database did not shut down normally");
			retStatus = DA_DOWNERR;
		} else  {
			System.out.println("Database shut down normally");
		}

		return retStatus;		
	}
	
	public ArrayList<PairInfo> getPairs() {
		ArrayList<PairInfo> list = new ArrayList<PairInfo>();
		String wordA = null;
		String wordB = null;
		PairInfo pair = null;
		try {
			resSet = s.executeQuery("SELECT * FROM PAIRS ORDER BY WORDLANGUAGEA");
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}
		try {
			while (resSet.next()) {
				wordA = resSet.getString(1);
				wordB = resSet.getString(2);
				pair = new PairInfo(wordA, wordB);
				list.add(pair);
			}
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}
		return list;
	}
	
	public ErrorInfo addPair(String wordA, String wordB) {
        // Check that word A is shorter than 128 characters.
        if ((wordA.length() > 128) | (wordB.length() > 128)) {
        	ErrorInfo ei = new ErrorInfo(DA_WORDTOOLONG, 
        							     resReview.getString("ERR_tooLong"));
        	return ei;
        }
        // Now, try to insert data into PAIRS table.
        try {
        	String stmtStr = "INSERT INTO PAIRS VALUES ('" +
			                 filterOutQuotes(wordA) + "', " +
			                 "'" + filterOutQuotes(wordB) + "')";
        	System.out.println(stmtStr);
        	s.execute(stmtStr);
        } catch (SQLException e) {
        	// Forget the insert attempt.
			try {
				conn.rollback();
			} catch (SQLException er) {
				System.out.println(er.getMessage());
			}
        	ErrorInfo ei = new ErrorInfo(DA_PAIRINSERTERR, e.getMessage());
        	return ei;
        }
        // At this point, let's commit.
		try {
			conn.commit();
		} catch (SQLException e) {
			ErrorInfo ei = new ErrorInfo(DA_COMMITERR, e.getMessage());
			return ei;
		}
		//
        ErrorInfo ei = new ErrorInfo(DA_SUCCESS, "");
		return ei;
	}
	
	public ErrorInfo updatePair(String previousWordA, String previousWordB,
			                    String wordA, String wordB) {
		// This could be optimized, depending on which word(s) was/were modified.
		ErrorInfo ei = deletePair(previousWordA, previousWordB);
		if (ei.getValue() != DA_SUCCESS) {
			return ei;
		}
		ei = addPair(wordA, wordB);
		return ei;
	}
	
	public ErrorInfo deletePair(String wordA, String wordB) {
		try {
			s.execute("DELETE FROM PAIRS WHERE WORDLANGUAGEA = '" + filterOutQuotes(wordA) +
					  "' AND WORDLANGUAGEB = '" + filterOutQuotes(wordB) + "'");
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			ErrorInfo ei = new ErrorInfo(DA_PAIRDELERR, e.getMessage());
			return ei;
		}
        // At this point, let's commit.
		try {
			conn.commit();
		} catch (SQLException e) {
			ErrorInfo ei = new ErrorInfo(DA_COMMITERR, e.getMessage());
			return ei;
		}
		//
		ErrorInfo ei = new ErrorInfo(DA_SUCCESS, "");
		return ei;
	}
	
	/*
	 * Error codes:
	 * - DA_SUCCESS
	 * - DA_GETRANDOMERR
	 * - DA_NOROWERR
	 */
	public ErrorInfo getRandomPair(PairInfo pairInfo) {
		
		int nbOfRows = 0;
		int rowNumber = 0;
		
		// First, get number of rows of PAIRS table.
		try {
		  resSet = s.executeQuery("SELECT COUNT(WORDLANGUAGEA) FROM PAIRS");
		  if (resSet.next()) {
		    nbOfRows = resSet.getInt(1);
		    System.out.println("number of rows: " + nbOfRows);
		  }
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			ErrorInfo ei = new ErrorInfo(DA_GETRANDOMERR, e.getMessage());
			return ei;			
		}
		if (nbOfRows == 0) {
			ErrorInfo ei = new ErrorInfo(DA_NOROWERR, resReview.getString("ERR_no_row"));
			return ei;
		}
		
		// Select a random integer from 0 up to number of rows less 1.
		rowNumber = pairsRandom.nextInt(nbOfRows);
		System.out.println("random row number: " + rowNumber);
		
		// Get words pair for generated random int.
		try {
			resSet = s.executeQuery("SELECT * FROM PAIRS OFFSET " + rowNumber + " ROWS " +
					                "FETCH NEXT 1 ROW ONLY");
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			ErrorInfo ei = new ErrorInfo(DA_GETRANDOMERR, e.getMessage());
			return ei;			
		}
		try {
			if (resSet.next()) {
				pairInfo.setWordLanguageA(resSet.getString(1));
				pairInfo.setWordLanguageB(resSet.getString(2));
				ErrorInfo ei = new ErrorInfo(DA_SUCCESS, "");
				return ei;
			} else {
			ErrorInfo ei = new ErrorInfo(DA_NOROWERR, resReview.getString("ERR_no_row"));
			return ei;			
			}
		} catch (SQLException e) {
			System.out.println(e.getMessage());
			ErrorInfo ei = new ErrorInfo(DA_GETRANDOMERR, e.getMessage());
			return ei;			
		}

	}
		
}