/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

package com.monblocnotes.review;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class HelpDialog extends Dialog {
	
	private static final int BUTTON_HEIGHT = 30;
	private static final int BUTTON_LENGTH = 100;

	private Shell shell = null;
	private Shell parent = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	
	private static final String VERSION_NUMBER = "V0.4";
	

	public HelpDialog(Shell parent, int style) {
		super(parent);	
	}
	
	public HelpDialog(Shell parent) {
		super(parent);
	}
	
	private String getModifications(String fileName) {
		
		// Because of Java Web Start, we need to reference resource files 
		// using dedicated class loader method.
		ClassLoader cl = this.getClass().getClassLoader(); 
		InputStream inputStream = cl.getResourceAsStream(fileName);
		System.out.println("trying to open modifications file " + fileName);
		if (inputStream == null) {
			System.out.println("null inputStream");
			return "";
		}
		
		StringBuilder contents = new StringBuilder();
	    
	    try {
	      BufferedReader inputReader =  new BufferedReader(
	    		  // TODO How to decide about the right character set?
	    		  new InputStreamReader (inputStream, "ISO-8859-1"));
	      try {
	        String line = null;
	        while ((line = inputReader.readLine()) != null){
	          contents.append(line);
	          contents.append(System.getProperty("line.separator"));
	        }
	      }
	      finally {
	        inputReader.close();
	      }
	    }
	    catch (IOException e){
	    	System.out.println(e.getMessage());
	    	contents.append(resReview.getString("ERR_modifications"));
	    }
	    
	    return contents.toString();
	}

	public void prepareDialog() {
		
		parent = getParent(); 
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(resReview.getString("AW_about1") + " - " + VERSION_NUMBER);
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		
		// Apply a one-column grid layout to the shell.
		GridLayout aboutGL = new GridLayout(1, false);
		shell.setLayout(aboutGL);
		
		// Display info. 
		Label infoLbl = new Label(shell, SWT.CENTER);
		infoLbl.setText(resReview.getString("AW_about2"));
		
		// Display list of modifications.
		Label modificationsLbl = new Label(shell, SWT.CENTER);
		modificationsLbl.setText(resReview.getString("AW_modifications"));
	    Text modificationsTxt = new Text(shell, SWT.MULTI | SWT.BORDER | SWT.WRAP | 
	    		                         SWT.V_SCROLL | SWT.READ_ONLY);
	    modificationsTxt.setLayoutData(new GridData(GridData.FILL_BOTH));
	    String modificationsStr = getModifications(resReview.getString("AW_modifications_file"));
	    modificationsTxt.setText(modificationsStr);
		
		// Prepare grid data, that will be used to center exit button, and to give it required size.
		GridData buttonGD = new GridData(BUTTON_LENGTH, BUTTON_HEIGHT);
		buttonGD.horizontalAlignment = SWT.CENTER;

		// Create exit button.
		Button exitBtn = new Button(shell, SWT.PUSH);
		exitBtn.setText(resReview.getString("AW_exit"));
		// Center it.
		exitBtn.setLayoutData(buttonGD);
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}
		});

	}
	
	public void open() {
		
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}

	}

}
