/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/
package com.monblocnotes.review;

import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/*
 * When application is delivered through Java Web Start, MessageBox does not
 * work correctly: it's not possible to close the dialog window! I'm quite sure
 * that's a problem in the architecture of my application... But, for now, I'll
 * implement my own message box...
 */
public class MyMessageBox extends Dialog {
	
	private static final int BUTTON_HEIGHT = 30;
	private static final int BUTTON_LENGTH = 100;

	private Shell shell = null;
	private Shell parent = null;
	//private int style = 0;
	private String text = "";
	private String message = "";
	
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");

	/*
	 * Implemented styles, for now:
	 * - SWT.OK
	 */
	public MyMessageBox(Shell parent, int style) {
		super(parent);	
		this.parent = parent;
		//this.style = style;
	}
	
	public MyMessageBox(Shell parent) {
		super(parent);
		this.parent = parent;
	}
	
	public void setMessage(String string) {
		message = string;
	}
	
	public void setText(String string) {
		text = string;
	}
	
	public int open() {
		
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(text);
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		// Apply a one-column grid layout to the shell.
		GridLayout messageBoxGL = new GridLayout(1, false);
		shell.setLayout(messageBoxGL);
		
		// Display message. 
		Label messageBoxLbl = new Label(shell, SWT.CENTER);
		messageBoxLbl.setText(message);

		// Prepare grid data, that will be used to center exit button, and to give it 
		// required size.
		GridData buttonGD = new GridData(BUTTON_LENGTH, BUTTON_HEIGHT);
		buttonGD.horizontalAlignment = SWT.CENTER;

		// Create exit button.
		Button exitBtn = new Button(shell, SWT.PUSH);
		exitBtn.setText(resReview.getString("AW_exit"));
		// Center it.
		exitBtn.setLayoutData(buttonGD);
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}
		});
		
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}

		return SWT.OK;
	}

}
