/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

package com.monblocnotes.review;

import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class ReviewLanguageXDialog extends Dialog {
	
	private final static int OUTPUT_FIELDS_LENGTH = 200;
	private static final int BUTTON_HEIGHT = 30;
	private static final int BUTTON_LENGTH = 200;
	
	private Shell shell = null;
	private Shell parent = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	private DataAccess dataAccess = null;
	
	// Language of word being reviewed: A or B.
	private char language = ' ';
	// Words being reviewed.
	private String word1 = null;
	private String word2 = null;
	
	private Text word1Txt = null;
	private Text word2Txt = null;
	
	public ReviewLanguageXDialog(Shell parent, int style) {
		super(parent);	
	}
	
	public ReviewLanguageXDialog(Shell parent) {
		super(parent);
	}
	
	// c must be either "A" or "B". 
	public ReviewLanguageXDialog(Shell parent, String c) {
		super(parent);
		if (c.equals("A")) {
			language = 'A';
		}
		else {
			language = 'B';
		}
	}
	
	public void refreshWords() {
		PairInfo pairInfo = new PairInfo("","");
		ErrorInfo ei = dataAccess.getRandomPair(pairInfo);
		if (ei.getValue() == DataAccess.DA_SUCCESS) {
			if (language == 'A') {
				word1 = pairInfo.getWordLanguageA();
				word2 = pairInfo.getWordLanguageB();
			} else {
				word1 = pairInfo.getWordLanguageB();
				word2 = pairInfo.getWordLanguageA();				
			}
		} else {
			// Usually: no words pairs stored yet. We should never get here, because
			// caller (Review) checks that there is no error, before activating
			// ReviewLanguageXDialog.
			word1 = new String("");
			word2 = new String("");
		}
	}

    public void prepareDialog(DataAccess dataAccessIn) {
    	
		dataAccess = dataAccessIn;
		parent = getParent(); 
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(resReview.getString("RLW_title"));
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		
		// Create a layout with 2 columns. Columns are not forced to have the same width.
		// Left column will contain the two words.
		// Right column will contain buttons.
		GridLayout reviewGL = new GridLayout(2, false);
		shell.setLayout(reviewGL);
		
		// Create a composite for words (first column).
		Composite wordsC = new Composite(shell, SWT.NONE);
		GridLayout wordsGL = new GridLayout(1, true);
		wordsC.setLayout(wordsGL);

		// Add first word. Can be of language A or of language B, depending on language field.
		word1Txt = new Text(wordsC, SWT.BORDER | SWT.READ_ONLY);
		// Force length of displayed word.
		GridData word1GD = new GridData();
		word1GD.widthHint = OUTPUT_FIELDS_LENGTH;
		word1Txt.setLayoutData(word1GD);
		// Add second word.
		word2Txt = new Text(wordsC, SWT.BORDER | SWT.READ_ONLY);
		// Force length of displayed word.
		GridData word2GD = new GridData();
		word2GD.widthHint = OUTPUT_FIELDS_LENGTH;
		word2Txt.setLayoutData(word2GD);
		// Second word is not visible, at startup.
		word2Txt.setVisible(false);
		
		// Prepare words for display.
		refreshWords();
		word1Txt.setText(word1);
		word2Txt.setText(word2);
		
		// Create a composite for buttons (second column).
		Composite buttonsC = new Composite(shell, SWT.NONE);
		// Give our requirements for positioning of buttons.
		GridData buttonsGD = new GridData();
		buttonsGD.horizontalAlignment = SWT.CENTER;
		buttonsC.setLayoutData(buttonsGD);
		// And apply a row layout.
		RowLayout buttonsRL = new RowLayout(SWT.VERTICAL);
		buttonsC.setLayout(buttonsRL);

		// Add buttons.
		// Display Translation button.
	    Button displayTranslationBtn = new Button(buttonsC, SWT.PUSH);
		displayTranslationBtn.setText(resReview.getString("RLW_display_translation"));
		displayTranslationBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		displayTranslationBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				word2Txt.setVisible(true);
				}
		});
		// Another Word button.
	    Button anotherWordBtn = new Button(buttonsC, SWT.PUSH);
		anotherWordBtn.setText(resReview.getString("RLW_another_word"));
		anotherWordBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		anotherWordBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				word2Txt.setVisible(false);
				refreshWords();
				word1Txt.setText(word1);
				word2Txt.setText(word2);				
				}
		});
		// Exit button.
	    Button exitBtn = new Button(buttonsC, SWT.PUSH);
		exitBtn.setText(resReview.getString("RLW_exit"));
		exitBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
				}
		});

    }
    
    public void open() {
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}   	
    }

}
